#!/bin/bash
#Create by lodder[at]delodder[dot]be.
SCRIPT="$0"
PACKAGE=time-drive

build-ui(){
    echo "Building ui ... "
    pyrcc4 ../ui/BackInTime_Icons.qrc -o ../timedrive/BackInTime_Icons_rc.py
}

build-locals(){
    echo "Building locals ..."
    for POFILE in ../po/*.po; do
        echo "Compiling $POFILE..."
        LANGCODE=$(basename $POFILE .po)
        OUTDIR=../locale/$LANGCODE/LC_MESSAGES/
        if [ ! -d $OUTDIR ]; then
            mkdir -p $OUTDIR
        fi
        msgfmt $POFILE -o $OUTDIR$PACKAGE.mo
    done
}

build-source(){
    echo "Building source ..."
    # Build Source Distribution from Distutils and setup.py
    rm -rf ../setup.py
    sed -e "s/%VERSION%/$VERSION/g" setup.py.template > ../setup.py
    cd ..
    python setup.py sdist
    cd dist
}

build-deb-unsigned(){
    echo "Building deb ... "
    mv time-drive-${VERSION}.tar.gz time-drive-${VERSION}.orig.tar.gz
    tar -xf time-drive-${VERSION}.orig.tar.gz
    
    # Copy Supporting Files
    cp -rf debian time-drive-${VERSION}/
    cp -rf PKG-INFO time-drive-${VERSION}/
    cd time-drive-${VERSION}
    
    # Build Deb Package
    # Todo: dpkg-buildpackage -us -uc builds the package without cryptographically signing it.  Such a package cannot be used in PPA.  Needs to be reworked so that the package is securely signed prior to upload.
    dpkg-buildpackage -us -uc 
    
    # Cleanup
    cd ..
    rm -rf time-drive-${VERSION} time-drive_${VERSION}-${PACKAGE_VERSION}.dsc time-drive_${VERSION}-${PACKAGE_VERSION}.tar.gz time-drive_${VERSION}-${PACKAGE_VERSION}_i386.changes time-drive-${VERSION}.orig.tar.gz
}

build-deb-signed(){
    echo "Building deb signed ..."
    mv time-drive-${VERSION}.tar.gz time-drive-${VERSION}.orig.tar.gz
    tar -xf time-drive-${VERSION}.orig.tar.gz
    
    # Copy Supporting Files
    cp -rf debian time-drive-${VERSION}/
    cp -rf PKG-INFO time-drive-${VERSION}/
    cd time-drive-${VERSION}
    
    # Build Deb Package
    
    dpkg-buildpackage -S -sa -k509F9033
    # Use -S option to create a source package 
    
    # Cleanup
    cd ..
    rm -rf time-drive-${VERSION}
}

upload-ppa(){
    echo "upload ppa ..."
    dput ppa:time-drive-devel/ppa time-drive_${VERSION}-${PACKAGE_VERSION}~${DIST}1_source.changes
    
    # Clean Up
    rm -rf time-drive_${VERSION}-${PACKAGE_VERSION}.dsc time-drive_${VERSION}-${PACKAGE_VERSION}_source.changes time-drive_${VERSION}-${PACKAGE_VERSION}_source.ppa.upload time-drive_${VERSION}-${PACKAGE_VERSION}.tar.gz time-drive-${VERSION}.orig.tar.gz
}

makeDebianFolder(){
    echo "setting changelog ..."
    rm -rf debian/changelog
    DATE=`date -R`
    sed -e "s/%DATE%/$DATE/g" -e "s/%PACKAGE%/$PACKAGE/g" -e "s/%DIST%/$DIST/g" -e "s/%PACKAGE_VERSION%/$PACKAGE_VERSION/g" -e "s/%VERSION%/$VERSION/g" changelog.template > debian/changelog
}

usage() {
    echo "-h or --help is for help"
    echo "--ui is to build the ui"
    echo "--source is to build the python sdist ./Build --source 0.3"
    echo "--signed is to build the signed debian example: ./Build.sh --signed 0.3 karmic 1"
    echo "--unsigned is to build unsigned debian example: ./Build.sh --unsigned 0.3 karmic 1"
    echo "--locals is to build locals"
    echo "--ppa uploads to ppa ./Build.sh --ppa 0.3 karmic 1"
}

numargs=$#

for ((i=1 ; i <= numargs ; i++)); do
  echo "$1"
  case "$1" in
    --source)
        echo "source option"
        if [ -z "$2" ] ; then
        (
          echo "$SCRIPT: --signed needs version"
        ) 1>&2
        exit 1
        fi
        export VERSION="$2"
        build-ui
        build-locals
        build-source
        shift 2
      ;;
    --ppa)
        echo "ppa option"
      if [ -z "$2" ] ; then
        (
          echo "$SCRIPT: --ppa needs version"
        ) 1>&2
        exit 1
      fi
      if [ -z "$3" ] ; then
        (
          echo "$SCRIPT: --ppa needs version and dist"
        ) 1>&2
        exit 1
      fi
      if [ -z "$4" ] ; then
        (
          echo "$SCRIPT: --ppa needs version and dist and package version"
        ) 1>&2
        exit 1
      fi
        upload-ppa
	shift 4
      ;;
    --ui)
        build-ui
      ;;
    --locals)
        build-locals
      ;;
    --signed)
      if [ -z "$2" ] ; then
        (
          echo "$SCRIPT: --signed needs version"
        ) 1>&2
        exit 1
      fi
      if [ -z "$3" ] ; then
        (
          echo "$SCRIPT: --signed needs version and dist"
        ) 1>&2
        exit 1
      fi
      if [ -z "$4" ] ; then
        (
          echo "$SCRIPT: --signed needs version and dist and package version"
        ) 1>&2
        exit 1
      fi
      export VERSION="$2"
      DIST="$3"
      PACKAGE_VERSION="$4"
      build-ui
      build-locals
      build-source
      makeDebianFolder
      build-deb-signed
      #move 3 parameters
      shift 4
      ;;
    --unsigned)
      if [ -z "$2" ] ; then
        (
          echo "$SCRIPT: --unsigned needs version"
        ) 1>&2
        exit 1
      fi
      if [ -z "$3" ] ; then
        (
          echo "$SCRIPT: --unsigned needs version and dist"
        ) 1>&2
        exit 1
      fi
      if [ -z "$4" ] ; then
        (
          echo "$SCRIPT: --unsigned needs version and dist and package version"
        ) 1>&2
        exit 1
      fi
      export VERSION="$2"
      DIST="$3"
      PACKAGE_VERSION="$4"
      build-ui
      build-locals
      build-source
      makeDebianFolder
      build-deb-unsigned
      shift 4
      ;;
    -h | --help)
      #print help
      usage
      #stop
      exit 0
      ;;
    --)
      #stop met het parsen van opties
      shift
      break
      ;;
    -*)
      #onbekende optie
      (
       usage
      ) 1>&2
      exit 1
      ;;
    *)
      #de rest is de gebruiker(s) die we moeten groeten
      break
      ;;
  esac
  shift
done
